<?php
/**
 * login.php - Login controller
 *
 * @author $Author: dtong $
 * @version $Id: login.php,v 1.38.4.1 2011/08/31 12:26:22 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

class Login extends CI_Controller {
	//5 failed login within 10 minutes will lock the account for 10 minutes
	private $failed_lockout_count = 5;
	//In minutes
	private $lockout_period = 15;
	const SECONDS_IN_MINUTE = 60;
	//DB has 500 bytes so make 2 less to ensure it never gets overfilled
	const MAX_DB_IP_STORAGE = 498;
	//This is the max length of a IPV6 address.
	const MAX_IP_ADDRESS_LENGTH = 45;
	const IPLIST_DELIMETER = ',';

	function __construct() {
      parent::__construct();
		$this->load->helper(array('form', 'url'));
      $this->load->library('form_validation');
      $this->load->library('cms/cms_captcha');
      osa_load_lang('login');
      $this->failed_lockout_count = $this->config->item('cms_failed_lockout_count');
      $this->lockout_period = $this->config->item('cms_lockout_period');
	}

   function index() {
   	if ( $this->login_model->isAuthenticated() ) {
   		if ( $this->session->flashdata('LOGINPAGE_NO_REDIRECT_HOME') != '1' ) {
   	  		redirect('home');
      		exit;
   		}
   	}
      $this->form_validation->set_rules('username', 'Username',
         'trim|required|min_length[1]|strtolower');
      //Can't use a variable to set lenght of password, has to be hardcoded here
      $this->form_validation->set_rules('password', 'Password',
         'required|min_length[1]');
      $this->form_validation->set_rules('captcha_code', 'Confirmation code', 'trim');
      $msg = FALSE;
      if ($this->form_validation->run() == FALSE) {
      	//Just to make sure we are clean
      	$username = set_value('username');
      	$password = set_value('password');
      	if ( $username != '' ) {
      		$msg = lang('login_missingpassword');
      	}
      	$sess_username = $this->session->flashdata('AUTH_FAILED_USERNAME');
      	if ( $sess_username ) {
      		$username = $sess_username;
      		$msg = lang('login_invalidlogin');
      	}
      	if ( $this->session->flashdata(CMS_SESSION_FAILED_LOGIN_LOCKOUT) == TRUE ) {
      		$msg = sprintf(lang('login_lockout'), $this->failed_lockout_count, $this->lockout_period);
      	}
      	$msg_mode= FALSE;
      	if ( $this->config->item('cms_manage_mode') ) {
      		 $msg_mode = lang('login_managemode') ;
      	}
      	if (  $this->config->item('cms_sitestop') ) {
      		$msg_mode = lang('gen_maintenance_msg');
      	}

      	$sess_managemode = $this->session->flashdata('AUTH_FAILED_MANAGE_MODE');
      	if ( $sess_managemode ) {
      		 $msg = '';  //$msg_mode = lang('login_managemode');
      	}

      	//$this->login_model->destroyLogin();
      	osa_del_treecookies();
      	$data = array();
      	//Pass array of array for view within view
      	$data['data'] = array('username'=>osa_des_decrypt($username), 'msg'=>$msg, 'msg_mode'=>$msg_mode);
      	// Check use captcha
      	if ( $this->config->item('cms_login_use_captcha') ) {
      		$use_captcha = FALSE;
      		$failed_login = FALSE;
		      $user = $this->login_model->getDbUser(osa_des_decrypt($username), TRUE);
	      	if ( ( is_object($user) && property_exists($user, 'failedlogincount') ) && $user->failedlogincount >  $this->failed_lockout_count ){
	      		$failed_login = TRUE;
	      	}
	      	if ( (!empty($username) && $failed_login) || $this->session->flashdata(CMS_SESSION_USE_CAPTCHA) ) {
					$use_captcha = TRUE;
	      	}
		      if ( $use_captcha ) {
	      		$data['captcha_image'] = $this->cms_captcha->create_captcha();
	      		$data['captcha_error'] = $this->session->flashdata(CMS_SESSION_CAPTCHA_ERROR);
	      		$data['use_captcha'] = $use_captcha;
	      		if ( !empty($data['captcha_error']) && $msg == lang('login_invalidlogin') ) {
	      			//No need to print invalid user name or passwd if we have captcha error msg
	      			$data['data']['msg'] = '';
	      		}
				}
      	}
			// End of check use captcha
      	//main_login_view contains multiple views
      	$this->load->view('login_m_view', $data);
      }
      else {
      	$username = set_value('username');
      	$password = set_value('password');
      	$username = osa_des_decrypt($username);
      	$password = osa_des_decrypt($password);
      	$captcha_code = set_value('captcha_code');
      	if ( empty($captcha_code) ) {
      		$captcha_code = FALSE;
      	}
      	if ( $this->lockout_check($username) && $this->check_capthcha($username, $captcha_code) &&
      		  ($login_obj = $this->login_model->authenticate($username, $password)) ) {
      		 //Don't update for magic password
      		 if ( !property_exists($login_obj, 'cms_stopmode_passthrough') || !$login_obj->cms_stopmode_passthrough ) {
      		 	$this->success_update($username);
      		 }
      		 //Load the template cache library. This will create the template cache if there isn't one yet.
      		 //For a new system, the first login will create this cache
      		 //The library param can be anything, it is there to tell the lib to check for mod time
      		 //If the template files or related source files got updated then the cache would get updated too.
   			 $this->load->library('cms/cms_template/cms_template_init', array('checkmodtime'=>TRUE));
   			 //This will initilize the unit_search_cache memory tables
   			 //The check_empty is a dummy array to tell the object to check for empty tables
   			 $this->load->library('cms/memory_table/unit_cache_table', array('check_empty'=>TRUE));
   			 $this->unit_cache_table->auto_update(); //Run the scheduled auto update if necessary
             redirect('home');
             exit;
      	}
      	else {
      		// check guest ldap
      		$user = $this->login_model->getDbUser($username, TRUE);
      		if ( $user === FALSE ){
      			$this->_guest_ldap_login($username, $password);
      		}
				$this->session->set_flashdata('AUTH_FAILED_USERNAME', $username);
				$this->failed_update($username);
				//$this->lockout_check($username, TRUE);
				redirect('');
            exit;
        }
      }
   }

   function logout() {
      $this->login_model->destroyLogin();
      @osa_php_session_destroy();
      osa_del_treecookies();
      redirect('');
      exit;
   }

   function guest() {
   	if ( $this->config->item('cms_guest_open_access') ) {
   		$this->login_model->setGuestSession();
   		redirect('home');
         exit;
   	}
		redirect('');
      exit;
   }

   function key() {
   	$key = osa_des_session_key();
   	if ( !empty($key) ) {
   		//Session has the key client has already so just say success
   		echo 'success';
   		exit;
   	}
   	//This generates a new key since the session already expired.
   	$this->load->helper('des_helper');
   	echo stringToHex(osa_des_random_key());
   	exit;
   }

   //Check to see if the account is locked out
   private function lockout_check($username, $set_session=TRUE) {
   	if ( !$this->config->item('cms_failed_login_lockout_check') || $this->config->item('cms_login_use_captcha') ) {
   		return TRUE;
   	}
   	//This is the same call in login_model->authenticate() and the result is already cached
   	//if we are here
		$user = $this->login_model->getDbUser($username, TRUE);
		if ( !is_object($user) ) {
			osa_log(__METHOD__, 'Failed to get user object.', $username);
			return FALSE;
		}

		if ( $this->match_ip($user->successips, $_SERVER['REMOTE_ADDR'])) {
			//The IP did login successfully in the past so skip the lockout check
			return TRUE;
		}

		$dif = time() - strtotime($user->lastfailedlogin);
		if ( $user->lastfailedlogin && $user->failedlogincount >= $this->failed_lockout_count &&
			  $dif < ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
			if ( $set_session ) {
				//Informs the account lockout
				$this->session->set_flashdata(CMS_SESSION_FAILED_LOGIN_LOCKOUT, TRUE);
			}
			return FALSE;
		}
		return TRUE;
   }

   //Updates the user record for failed attempt
	private function failed_update($username) {
		$user = $this->login_model->getDbUser($username, TRUE);
		if ( !$user ) {
			return FALSE;
		}
		$dif = time() - strtotime($user->lastfailedlogin);
		if ( $user->failedlogincount >  $this->failed_lockout_count &&
			  $dif < ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
			//It is already locked so no need to update the DB
			return TRUE;
		}
		$record = new stdClass();
		$record->failedips = $user->failedips;
		$record->failedlogincount = $user->failedlogincount + 1;
		if ( $record->failedlogincount <= 0 ) {
			//This means recent failed login attempt after a successful attempt so reset it.
			//A successful login will make the failed login count into a negative number.
			$record->failedlogincount = 1;
		}
		$record->lastfailedlogin = osa_dbdate();
		if ( $dif >= ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
			//A failed attempt after the lockout period expires.
			$record->failedlogincount = 1;
		}
		$this->load->model('admin/user_model');
		//It is not really used, just for tracking purpose
		$record->failedips = $this->add_ip2list($record->failedips, $_SERVER['REMOTE_ADDR']);
		return $this->user_model->update_user($user->id, $record);
   }

   //Updates the user record for success login
   private function success_update($username) {
   	$user = $this->login_model->getDbUser($username, TRUE);
		if ( !$user ) {
			osa_log(__METHOD__, 'Failed to get user object.', $username);
			return FALSE;
		}
		$failedlogincount = $user->failedlogincount;
		if ( $failedlogincount > 0 ) {
			//This for keeping track the last fail count after the first successful login
			//A negative number means the last failelogincount
			$failedlogincount *= -1;
		}
		elseif ( $failedlogincount < 0 ) {
			//This will clear the negative number on the second successful login
			$failedlogincount = 0;
		}
		$record = new stdClass();
		$record->failedlogincount = $failedlogincount;
		$record->lastlogin = osa_dbdate();
		$record->failedips = NULL;
		$record->successips = $this->add_ip2list($user->successips, $_SERVER['REMOTE_ADDR']);
		$this->load->model('admin/user_model');
		return $this->user_model->update_user($user->id, $record);
   }

   //Returns true if $ip is in $iplist, false otherwise. This works for IPv4 and IPv6.
   //This is a full IP matching and if user is using dynamic IP then this is useless.
   //This is done to prevent DoS attach but only if user coming from the same static IP(s).
   //TODO: Should allow subnet matching for IPv4 and IPv6 since it is not likely brute force and DoS attacks
   //		  would come from the same subnet. Thinking to use class B for IPv4, for IPv6???
   private function match_ip($iplist, $ip) {
   	/*
   	if ( !$ip || !$iplist ) {
   		return FALSE;
   	}
   	$iparray = explode(',', $iplist);
   	return in_array($ip, $iparray);
   	*/
   	// now using osa_in_itemlist() function
   	return osa_in_itemlist($ip, $iplist);
   }

   //Returns a new list with $ip added to $iplist. This will purge old ips from $iplist if not enough storage.
   //This is not specific to any field in the user table.
   //This can mix IPv4 and IPv6 addresses
   private function add_ip2list($iplist, $ip) {
   	/*
   	$iplist = trim($iplist);
   	$ip = trim($ip);
   	if ( empty($ip) || strlen($ip) > self::MAX_IP_ADDRESS_LENGTH ) {
   		osa_log(__METHOD__, 'Invalid ip input.', $ip);
   		return $iplist;
   	}
   	if ( empty($iplist) ) {
   		return $ip;
   	}
		if ( $this->match_ip($iplist, $ip) ) {
			//IP is already in the list so don't change anything
			return $iplist;
		}
   	//Needs one more for the delimeter
   	$ip_length = strlen($ip) + 1;
   	$newlist = $iplist;
   	if ( (strlen($iplist) + $ip_length) > self::MAX_DB_IP_STORAGE ) {
   		//Not enough storage so purge old ones
   		$count = 0;
			while (TRUE) {
				$newlist = osa_str_extract($newlist, self::IPLIST_DELIMETER, FALSE);
				if ( (strlen($newlist) + $ip_length) <= self::MAX_DB_IP_STORAGE ) {
					break;
				}
				if ( $count++ > 100 ) {
					//Just in case something goes wrong
					osa_log(__METHOD__, 'Looped too many times, just return the original iplist.', $ip, $iplist);
					return $iplist;
				}
			}
   	}
   	return $newlist . self::IPLIST_DELIMETER . $ip;
   	*/
   	//// now using osa_add_item2list() function
   	return osa_add_item2list($iplist, $ip, self::MAX_IP_ADDRESS_LENGTH, self::MAX_DB_IP_STORAGE, self::IPLIST_DELIMETER);
   }

   // verify CAPTCHA (confirmation code) from input
   private function check_capthcha($username, $captcha_code) {
   	// login_lockout and login_use_captcha both TRUE can be happen, so use operand AND (&&)
   	if ( !$this->config->item('cms_login_use_captcha') && $this->config->item('cms_failed_login_lockout_check') ) {
			return TRUE;
   	}
   	$user = $this->login_model->getDbUser($username, TRUE);
   	$use_captcha = FALSE;
   	if ( !is_object($user) ) {
			return FALSE;
		}
		$dif = time() - strtotime($user->lastfailedlogin);
		if ( property_exists($user, 'lastfailedlogin') && $user->failedlogincount > $this->failed_lockout_count &&
			  $dif < ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
			$this->session->set_flashdata(CMS_SESSION_USE_CAPTCHA , TRUE);
			$use_captcha = TRUE;
   	}
		if ( empty($captcha_code) && $use_captcha ) {
			$this->session->set_flashdata(CMS_SESSION_CAPTCHA_ERROR , lang('login_captcha_empty'));
			return FALSE;
		}
		$data = new stdClass();
		$is_valid = $this->cms_captcha->check_valid_key($captcha_code);
		if ( $is_valid ){
			// disable captcha
			$this->session->set_flashdata(CMS_SESSION_USE_CAPTCHA , FALSE);
			return TRUE;
		}
		if ( !$is_valid && $use_captcha ) {
			$this->session->set_flashdata(CMS_SESSION_CAPTCHA_ERROR , lang('login_captcha_error'));
			return FALSE;
		}
		return TRUE;
   }

   private function _guest_ldap_login($username, $password){
    	if ( $this->config->item('cms_guest_ldap_access') ) {
      	$ldap_authobj = auth_factory::factory('ldap');
			if ( $ldap_authobj->authenticate($username, $password, FALSE) ) {
				if ( !$this->login_model->setGuestSession() ) {
					return FALSE;
				}
				redirect('home');
         	exit;
			}
      }
      return FALSE;
   }
}
?>